<?php

namespace WPRA;

use WPRA\Helpers\AjaxFactory;
use WPRA\Integrations\Cache;
use WPRA\Integrations\Woo;
use WPRA\Helpers\Utils;
use WPRA\Database\Query;
use wpdb;

class App {

	/** @var wpdb */
	static public $db;
	static public $wp_version;
	private $license;
	private $addon;
	private $ajax;
	private static $_instance = null;

	function __construct() {
		self::$db         = $GLOBALS['wpdb'];
		self::$wp_version = $GLOBALS['wp_version'];

		Config::initialize();
		Migrations::handle();
		Update::init();
		Enqueue::init();;
		AdminPages::init();
		Metaboxes::render();
		Shortcode::register();
		Woo::init();
		Cache::init();

		$this->license = License::instance();
		$this->addon   = Addon::instance();
		$this->ajax    = Ajax::init( 'wpra' );

		$this->addAdminHeadScripts();
		$this->registerHooks();
	}

	public static function run() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}

		return self::$_instance;
	}

	private function registerHooks() {
		register_activation_hook( WPRA_PLUGIN_PATH . 'wpreactions-pro.php', [ $this, "activation" ] );
		register_deactivation_hook( WPRA_PLUGIN_PATH . 'wpreactions-pro.php', [ $this, "deactivation" ] );
		add_filter( 'the_content', [ $this, 'content_adder' ], 9999 );
		add_action( 'init', [ $this, 'init' ], 20 );
		add_filter( 'plugin_action_links_' . WPRA_PLUGIN_BASENAME, [ $this, 'action_links' ] );
		add_action( 'admin_notices', [ $this, 'lite_deactivation_notice' ] );
		add_action( 'deleted_post', [ $this, 'onPostDelete' ] );

		add_filter( 'admin_body_class', function ( $cls ) {
			return $cls . ' wpreactions-admin ';
		} );

		add_action( 'admin_footer', function () {
			if ( ! Utils::isWpraAdmin() ) return;

			Utils::renderTemplate( 'view/admin/components/floating-menu' );
			Utils::renderTemplate( 'view/admin/components/modals/feedback-form' );
			Utils::renderTemplate( 'view/admin/components/modals/reset-emoji-stats' );
			Utils::renderTemplate( 'view/admin/components/modals/revoke-license' );
			Utils::renderTemplate( 'view/admin/components/modals/reset-global-options' );
		} );
	}

	// run necessary actions on plugin activation
	function activation() {
		// does not support above wp version 4.7
		if ( version_compare( self::$wp_version, '4.7', '<' ) ) {
			deactivate_plugins( WPRA_PLUGIN_BASENAME );
			ob_start(); ?>
            <p style="color: red;">WP Reactions Pro requires WordPress version 4.7 or above. Please upgrade your WordPress and try again.</p>
            <p><a href="<?php echo admin_url( 'plugins.php' ); ?>">Back to Plugins</a></p>
			<?php wp_die( ob_get_clean() );
		}

		if ( is_plugin_active( 'wp-reactions-lite/wp-reactions-lite.php' ) ) {
			deactivate_plugins( 'wp-reactions-lite/wp-reactions-lite.php' );
			set_transient( 'wpra-lite-plugin-disabled-notice', true, 5 );
		}

		Activation::start();
	}

	function deactivation() {
		// schedule event
		$timestamp = wp_next_scheduled( 'wpra_check_updates' );
		wp_unschedule_event( $timestamp, 'wpra_check_updates' );

		// remove caps for the plugin
		$role = get_role( 'administrator' );
		if ( $role !== null ) {
			$role->remove_cap( 'access_wpreactions' );
			$role->remove_cap( 'edit_wpreactions' );
		}

		if ( defined( 'WPRA_MRU_PLUGIN_BASENAME' ) ) {
			deactivate_plugins( WPRA_MRU_PLUGIN_BASENAME );
		}
	}

	function init() {
		load_plugin_textdomain( 'wpreactions', false, dirname( WPRA_PLUGIN_BASENAME ) . '/languages/' );
		do_action( 'wpreactions/plugin/initialized' );
	}

	function addAdminHeadScripts() {
		// Show some messages while all emojis gets loaded
		add_action( 'admin_head', function () { ?>
            <script type="text/javascript">
                setTimeout(function () {
                    jQuery('.overlay-message').html('We are almost done...');
                }, 3000);

                setTimeout(function () {
                    jQuery('.overlay-message').html('Finalizing...');
                }, 6000);
            </script>
			<?php
		} );
	}

	// purge data on post delete
	static function onPostDelete( $post_id ) {
		self::$db->delete(
			Config::$tbl_reacted_users,
			[ 'bind_id' => $post_id ]
		);

		self::$db->delete(
			Config::$tbl_social_stats,
			[ 'bind_id' => $post_id ]
		);
	}


	function content_adder( $content ) {
		global $post;

		// Give it to Woo integration
		if ( $post->post_type == 'product' ) {
			return $content;
		}

		$deploys   = Config::getGlobalDeployments();
		$post_type = $post->post_type;

		// add emojis if only we are in main query and looping
		// TODO: check if this has any effect
		if ( ! ( in_the_loop() and is_main_query() ) ) {
			return $content;
		}

		$allow_emojis = Utils::get_post_meta( $post->ID, '_wpra_show_emojis' );

		if ( $allow_emojis == 'false' ) {
			return $content;
		}

		// if post_type bound to any sgc, then use it
		$sgc = Shortcode::getSgcDataBy( 'post_type', $post_type, [ 'id', 'front_render' ] );
		if ( ! is_null( $sgc ) && $sgc->front_render ) {
			$reactions = Shortcode::handle( [ 'bind_to_post' => 'yes', 'sgc_id' => $sgc->id ] );

			return $content . $reactions;
		}

		// not activated globally
		if ( ! Config::isGlobalActivated() ) {
			return $content;
		}

		// post type is not deployed auto or manually
		if ( ! Config::isGloballyDeployed( $post_type ) ) {
			return $content;
		}

		if ( in_array( $post_type, $deploys['manual'] ) && $allow_emojis != 'true' ) {
			return $content;
		}

		$before    = $after = '';
		$reactions = Shortcode::handle();

		if ( Config::$active_layout_opts['content_position'] == 'before' ) {
			$before = $reactions;
		} else if ( Config::$active_layout_opts['content_position'] == 'after' ) {
			$after = $reactions;
		} else {
			$before = $after = $reactions;
		}

		return $before . $content . $after;
	}

	static function getFakeCounts( $bind_id ) {
		$fake_counts = get_post_meta( $bind_id, '_wpra_start_counts', true );
		if ( is_array( $fake_counts ) and ! empty( $fake_counts ) ) {
			return array_map( 'intval', $fake_counts );
		}

		return [];
	}

	static function getCountsTotal( $bind_id, $fake_counts, $emojis ) {
		if ( empty( $fake_counts ) ) {
			$fake_counts = self::getFakeCounts( $bind_id );
		}

		$db_counts = self::getReactionCountsPerEmoji( $bind_id, $emojis );

		$result = [];
		foreach ( $emojis as $emoji_id ) {
			$fake_count          = isset( $fake_counts[ $emoji_id ] ) ? intval( $fake_counts[ $emoji_id ] ) : 0;
			$db_count            = isset( $db_counts[ $emoji_id ] ) ? intval( $db_counts[ $emoji_id ] ) : 0;
			$result[ $emoji_id ] = $fake_count + $db_count;
		}

		return $result;
	}

	static function getReactionCountsPerEmoji( $bind_id, $emojis ) {

		$counts = Query\Select
			::create()
			->fields( [ 'emoji_id', 'count(*) as count' ] )
			->table( Config::$tbl_reacted_users )
			->where( [ 'bind_id' => $bind_id, 'emoji_id' => [ 'in', $emojis ] ] )
			->groupBy( 'emoji_id' )
			->output( ARRAY_A )
			->run();

		return array_column( $counts, 'count', 'emoji_id' );
	}

	static function getFakeShareCounts( $post_id, $default = 0 ) {
		$fake_share_count = get_post_meta( $post_id, '_wpra_fake_share_count', true );

		return $fake_share_count == '' ? $default : intval( $fake_share_count );
	}

	static function getUserReactedEmoji( $bind_id ) {
		$already = '';
		if ( Config::$settings['user_reaction_limitation'] == 1 && isset( $_COOKIE['react_id'] ) ) {
			$already = Query\Select
				::create()
				->fields( 'emoji_id' )
				->table( Config::$tbl_reacted_users )
				->where( [ 'bind_id' => $bind_id, 'react_id' => $_COOKIE['react_id'] ] )
				->one()
				->run();
		}

		return $already;
	}

	static function getTotalShareCounts( $bind_id, $fake_count ) {
		$fake_counts = empty( $fake_count ) ? self::getFakeShareCounts( $bind_id ) : $fake_count;
		$real_counts = self::getUserShareCounts( $bind_id );

		return Utils::formatCount( $real_counts + $fake_counts );
	}

	static function getUserShareCounts( $bind_id ) {
		return Query\Select
			::create()
			->fields( 'count(*)' )
			->table( Config::$tbl_social_stats )
			->where( [ 'bind_id' => $bind_id ] )
			->one()
			->run();
	}

	static function getSocialSharePerPlatform( $bind_id ) {
		$data = Query\Select
			::create()
			->fields( [ 'platform', 'count(*) as clicks' ] )
			->table( Config::$tbl_social_stats )
			->where( [ 'bind_id' => $bind_id ] )
			->groupBy( 'platform' )
			->run();

		return array_column( $data, 'clicks', 'platform' );
	}

	function action_links( $links ) {
		$links[] = '<a href="' . Utils::getAdminPage( 'dashboard' ) . '">' . __( 'Settings', 'wpreactions' ) . '</a>';
		$links[] = '<a target="_blank" href="https://wpreactions.com/support">' . __( 'Support', 'wpreactions' ) . '</a>';

		return $links;
	}

	function lite_deactivation_notice() {
		if ( get_transient( 'wpra-lite-plugin-disabled-notice' ) ) { ?>
            <div class="notice-warning notice is-dismissible">
                <p><strong>WP Reactions Lite</strong> version disabled due to WP Reactions Pro version activation.</p>
            </div>
			<?php
			delete_transient( 'wpra-lite-plugin-disabled-notice' );
		}
	}

	static function current_user_can( $caps ) {
		if ( empty( $caps ) ) return true;

		$user = wp_get_current_user();
		if ( ! is_array( $caps ) ) {
			return $user->has_cap( $caps );
		}

		foreach ( $caps as $cap ) {
			if ( ! $user->has_cap( $cap ) ) {
				return false;
			}
		}

		return true;
	}

	function license() {
		return $this->license;
	}

	function addon() {
		return $this->addon;
	}

	function ajax() {
		return $this->ajax;
	}

} // end of WP Emoji class
