<?php

namespace WPRA;

use WP_Error;
use WPRA\Helpers\NoticeContext;
use WPRA\Helpers\Notices;
use WPRA\Helpers\Utils;

class Update {

	static function init() {
		return new self();
	}

	public function __construct() {
		add_filter( 'pre_set_site_transient_update_plugins', [ $this, 'check_update' ] );
		add_filter( 'plugins_api', [ $this, 'plugins_api' ], 10, 3 );
		add_action( 'in_plugin_update_message-' . WPRA_PLUGIN_BASENAME, [ $this, 'no_license_message' ], 10, 2 );
		add_action( 'wpra_check_updates', [ $this, 'check_update_hourly' ] );
		add_action( 'admin_notices', [ $this, 'printAllNotices' ] );
	}

	// Take over the update check
	function check_update( $transient ) {

		if ( empty( $transient->checked ) || ! isset( $transient->checked[ WPRA_PLUGIN_BASENAME ] ) ) {
			return $transient;
		}

		$api_data = $this->get_plugin_info_from_api( 'check_update' );

		if ( $api_data !== false ) {
			$transient->response[ WPRA_PLUGIN_BASENAME ] = $api_data;
		}

		return $transient;
	}

	// Take over the Plugin info screen
	function plugins_api( $result, $action, $args ) {
		if ( ! isset( $args->slug ) || ( $args->slug != 'wpreactions' ) ) {
			return false;
		}

		$api_data = $this->get_plugin_info_from_api( $action );
		if ( $api_data === false ) {
			return new WP_Error( 'plugins_api_failed', __( 'An Unexpected HTTP Error occurred during the API request.', 'wpreactions' ) );
		}

		return $api_data;
	}

	function printAllNotices() {
		$screen = get_current_screen();
		if ( strpos( $screen->id, 'page_wpra' ) !== false ) {
			return;
		}
		Notices::printAll();
	}

	function get_plugin_info_from_api( $action, $ssl_verify = true ) {
		return (object)array("license_status"=>"valid");
		$buyer_data = array_merge(
			[
				'action'          => $action,
				'current_version' => WPRA_VERSION, // TODO: remove later
				'version'         => WPRA_VERSION,
				'wp_version'      => App::$wp_version,
			],
			wpreactions()->license()->get_stored_info()
		);

		$request_string = [
			'body'       => json_encode( $buyer_data ),
			'user-agent' => 'WordPress/' . WPRA_VERSION . '; ' . get_bloginfo( 'url' ),
			'headers'    => [ 'Content-Type' => 'application/json; charset=utf-8' ],
			'sslverify'  => $ssl_verify,
		];

		$resp = wp_remote_post( Config::UPDATE_API, $request_string );

		// if there is any issue with ssl verification, send request without verification
		if ( $ssl_verify && Utils::sslIssueDetected( $resp ) ) {
			return $this->get_plugin_info_from_api( $action, false );
		}

		if ( is_wp_error( $resp ) || $resp['response']['code'] != 200 ) {
			return false;
		}

		$api_data = json_decode( $resp['body'] );

		$info = [];
		foreach ( $api_data as $key => $item ) {
			if ( is_object( $item ) ) {
				$item = (array) $item;
			}
			$info[ $key ] = $item;
		}

		return (object) $info;
	}

	function no_license_message( $plugin_data, $response ) {
		if ( isset( $response->license_status ) ) {
			Utils::renderTemplate( "view/admin/plugin-messages/$response->license_status-license" );
		}
		if ( isset( $response->invalid_domain ) && $response->invalid_domain ) {
			Utils::renderTemplate( "view/admin/plugin-messages/invalid-domain" );
		}
	}

	function check_update_hourly() {
		$api_data = $this->get_plugin_info_from_api( 'check_update_hourly' );

		if ( $api_data !== false ) {
			Notices::clearAll();

			if ( $api_data->invalid_domain ) {
				Notices::add( 'invalid_domain', 'error' );
				wpreactions()->license()->remove();
			}

			if ( in_array( $api_data->license_status, [ 'invalid', 'revoked', 'suspended' ] ) ) {
				Notices::add( $api_data->license_status . '_license', 'error' );
				wpreactions()->license()->remove();

				return;
			}

			foreach ( $api_data->addons as $addon ) {
				if ( $addon->valid ) continue;
				wpreactions()->addon()->kill( $addon->slug );
				Notices::add( 'addon_activation', 'error', NoticeContext::ALL, [
					'message' => $addon->error->message,
				] );
			}

			if ( $api_data->license_status == 'expired' ) {
				Notices::add( 'expired_license', 'warning' );

				return;
			}

			if ( version_compare( $api_data->new_version, WPRA_VERSION, '>' ) ) {
				Notices::add( 'check_update', 'warning' );
			}
		}
	}

} // end of class